package cache;

import cache.core.Person;
import cache.core.PersonRegistry;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ws.rs.Consumes;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.CacheControl;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.EntityTag;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;

import javax.ws.rs.core.Response.ResponseBuilder;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.UriInfo;

/**
 *
 * Same as PersonResource but using conditional GETs and conditional updates. 
 * All methods use Response objects to send custom HTTP response (400-599) 
 * (exception handling)
 *
 * @author hajo
 */
@Path("/cond")
public class PersonResourceConditional {

    private static final Logger LOG = Logger.getLogger(PersonResourceConditional.class.getName());
    private final static PersonRegistry reg = PersonRegistry.INSTANCE;
    // Helper class used to build URI's. Injected by container 
    @Context
    private UriInfo uriInfo;

   
    // --------------- CONDITIONAL GET ----------------
    @GET
    @Path("{id}")
    @Produces({MediaType.APPLICATION_JSON})
    public Response find(@PathParam("id") Long id, @Context Request request) {
        Person p = reg.find(id);
        if (p == null) {
            // return Response.noContent().build();
        }
        String s = ETagGenerator.getETagFor(p);
        EntityTag tag = new EntityTag(s);

        // Etag Will be evaluted agaist  If-None-Match
        // header in request. 
        // Returnvalues: 
        // --- null if preconditions ARE met
        // --- if NOT create a RespondeBuilder with appropriate status
        ResponseBuilder builder = request.evaluatePreconditions(tag);

        CacheControl cc = new CacheControl();
        cc.setMaxAge(10);  // sec.

        if (builder != null) {  // Precondition NOT met (If-None-Match is false)
            LOG.log(Level.INFO, "Builder NOT null");
            builder.cacheControl(cc);
            // Send 304 "Not Modified"      
            return builder.build();
        } else {
            LOG.log(Level.INFO, "Builder null");
            // Send representation of current data
            return Response.ok(p).tag(tag).cacheControl(cc).build();
        }

    }
    
    // ---------------- CONDITIONAL PUT  -------------------------
    @PUT
    @Path("{id}")
    @Consumes(MediaType.APPLICATION_FORM_URLENCODED)
    @Produces({MediaType.APPLICATION_JSON})
    public Response update(@PathParam("id") Long id, @FormParam("fname") String fname,
            @FormParam("age") int age, @Context Request request) {
        LOG.log(Level.INFO, "Update {0}", id);

        Person p = reg.find(id);
        String s = ETagGenerator.getETagFor(p);
        EntityTag tag = new EntityTag(s);

        ResponseBuilder builder = request.evaluatePreconditions(tag);
        if (builder != null) {  // Precondition NOT met (If-Match false)
            // Send 412 "precondition failed"     
            return builder.build();
        } else {
            try {
                Person old = reg.find(id);
                reg.update(new Person(id, fname, age));
                // Send Ok 200
                return Response.ok(old).build();
            } catch (IllegalArgumentException e) {
                return Response.status(Status.INTERNAL_SERVER_ERROR).build();
            }
        }
    }

}
